//! A type-erased wrapper around plotters::DrawingArea.
//! This is not exposed to Python user and only used internally.
//!
//! For now, it only supports (x,y) = (float,float)

struct Area {
}

impl Area {
}
